@@ -1,4 +1,4 @@
-/* $Id: thunar-sbr-number-renamer.c 20550 2006-03-25 21:52:38Z benny $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -50,8 +50,6 @@ enum
 
 
 
-static void   thunar_sbr_number_renamer_class_init    (ThunarSbrNumberRenamerClass  *klass);
-static void   thunar_sbr_number_renamer_init          (ThunarSbrNumberRenamer       *number_renamer);
 static void   thunar_sbr_number_renamer_finalize      (GObject                      *object);
 static void   thunar_sbr_number_renamer_get_property  (GObject                      *object,
                                                        guint                         prop_id,
@@ -65,7 +63,7 @@ static void   thunar_sbr_number_renamer_realize       (GtkWidget
 static gchar *thunar_sbr_number_renamer_process       (ThunarxRenamer               *renamer,
                                                        ThunarxFileInfo              *file,
                                                        const gchar                  *text,
-                                                       guint                         index);
+                                                       guint                         idx);
 static void   thunar_sbr_number_renamer_update        (ThunarSbrNumberRenamer       *number_renamer);
 
 
@@ -177,9 +175,9 @@ thunar_sbr_number_renamer_init (ThunarSbrNumberRenamer *number_renamer)
   GtkWidget      *entry;
   GtkWidget      *label;
   GtkWidget      *hbox;
-  gint            n;
+  guint           n;
 
-  hbox = gtk_hbox_new (FALSE, 12);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_start (GTK_BOX (number_renamer), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
@@ -187,10 +185,10 @@ thunar_sbr_number_renamer_init (ThunarSbrNumberRenamer *number_renamer)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
-  combo = gtk_combo_box_new_text ();
+  combo = gtk_combo_box_text_new ();
   klass = g_type_class_ref (THUNAR_SBR_TYPE_NUMBER_MODE);
   for (n = 0; n < klass->n_values; ++n)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(klass->values[n].value_nick));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _(klass->values[n].value_nick));
   exo_mutual_binding_new (G_OBJECT (number_renamer), "mode", G_OBJECT (combo), "active");
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
@@ -225,7 +223,7 @@ thunar_sbr_number_renamer_init (ThunarSbrNumberRenamer *number_renamer)
   atk_relation_set_add (relations, relation);
   g_object_unref (G_OBJECT (relation));
 
-  hbox = gtk_hbox_new (FALSE, 12);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 12);
   gtk_box_pack_end (GTK_BOX (number_renamer), hbox, FALSE, FALSE, 0);
   gtk_widget_show (hbox);
 
@@ -233,10 +231,10 @@ thunar_sbr_number_renamer_init (ThunarSbrNumberRenamer *number_renamer)
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
-  combo = gtk_combo_box_new_text ();
+  combo = gtk_combo_box_text_new ();
   klass = g_type_class_ref (THUNAR_SBR_TYPE_TEXT_MODE);
   for (n = 0; n < klass->n_values; ++n)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(klass->values[n].value_nick));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _(klass->values[n].value_nick));
   exo_mutual_binding_new (G_OBJECT (number_renamer), "text-mode", G_OBJECT (combo), "active");
   gtk_box_pack_start (GTK_BOX (hbox), combo, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), combo);
@@ -258,7 +256,7 @@ thunar_sbr_number_renamer_init (ThunarSbrNumberRenamer *number_renamer)
   gtk_widget_show (entry);
 
   label = gtk_label_new_with_mnemonic (_("_Text:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
   gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
   gtk_widget_show (label);
@@ -371,7 +369,7 @@ static gchar*
 thunar_sbr_number_renamer_process (ThunarxRenamer  *renamer,
                                    ThunarxFileInfo *file,
                                    const gchar     *text,
-                                   guint            index)
+                                   guint            idx)
 {
   ThunarSbrNumberRenamer *number_renamer = THUNAR_SBR_NUMBER_RENAMER (renamer);
   gboolean                invalid = TRUE;
@@ -404,26 +402,26 @@ thunar_sbr_number_renamer_process (ThunarxRenamer  *renamer,
   switch (number_renamer->mode)
     {
     case THUNAR_SBR_NUMBER_MODE_123:
-      number = g_strdup_printf ("%u", start + index);
+      number = g_strdup_printf ("%u", start + idx);
       break;
 
     case THUNAR_SBR_NUMBER_MODE_010203:
-      number = g_strdup_printf ("%02u", start + index);
+      number = g_strdup_printf ("%02u", start + idx);
       break;
 
     case THUNAR_SBR_NUMBER_MODE_001002003:
-      number = g_strdup_printf ("%03u", start + index);
+      number = g_strdup_printf ("%03u", start + idx);
       break;
 
     case THUNAR_SBR_NUMBER_MODE_000100020003:
-      number = g_strdup_printf ("%04u", start + index);
+      number = g_strdup_printf ("%04u", start + idx);
       break;
 
     case THUNAR_SBR_NUMBER_MODE_ABC:
       if (start >= 'a' && start <= 'z')
-        number = g_strdup_printf ("%c", (gchar) (MIN (start + index, 'z')));
+        number = g_strdup_printf ("%c", (gchar) (MIN (start + idx, 'z')));
       else if (start >= 'A' && start <= 'Z')
-        number = g_strdup_printf ("%c", (gchar) (MIN (start + index, 'Z')));
+        number = g_strdup_printf ("%c", (gchar) (MIN (start + idx, 'Z')));
       else
         g_assert_not_reached ();
       break;
@@ -473,13 +471,12 @@ thunar_sbr_number_renamer_update (ThunarSbrNumberRenamer *number_renamer)
   GdkColor back;
   GdkColor text;
   gchar   *endp;
-  guint    number;
 
   /* check whether "start" is valid for the "mode" */
   if (number_renamer->mode < THUNAR_SBR_NUMBER_MODE_ABC)
     {
       /* "start" must be a positive number */
-      number = strtoul (number_renamer->start, &endp, 10);
+      strtoul (number_renamer->start, &endp, 10);
       invalid = (endp <= number_renamer->start || *endp != '\0');
     }
   else if (number_renamer->mode == THUNAR_SBR_NUMBER_MODE_ABC)
@@ -491,12 +488,12 @@ thunar_sbr_number_renamer_update (ThunarSbrNumberRenamer *number_renamer)
     }
 
   /* check if the start entry is realized */
-  if (GTK_WIDGET_REALIZED (number_renamer->start_entry))
+  if (gtk_widget_get_realized (number_renamer->start_entry))
     {
       /* check if the "start" value is valid */
       if (G_UNLIKELY (invalid))
         {
-          /* if GTK+ wouldn't be that stupid with style properties and 
+          /* if GTK+ wouldn't be that stupid with style properties and
            * type plugins, this would be themable, but unfortunately
            * GTK+ is totally broken, and so it's hardcoded.
            */
