@@ -1,4 +1,4 @@
-/* $Id: thunar-sbr-remove-renamer.c 22062 2006-06-11 06:00:11Z daichi $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -40,8 +40,6 @@ enum
 
 
 
-static void   thunar_sbr_remove_renamer_class_init    (ThunarSbrRemoveRenamerClass  *klass);
-static void   thunar_sbr_remove_renamer_init          (ThunarSbrRemoveRenamer       *remove_renamer);
 static void   thunar_sbr_remove_renamer_get_property  (GObject                      *object,
                                                        guint                         prop_id,
                                                        GValue                       *value,
@@ -54,7 +52,7 @@ static void   thunar_sbr_remove_renamer_realize       (GtkWidget
 static gchar *thunar_sbr_remove_renamer_process       (ThunarxRenamer               *renamer,
                                                        ThunarxFileInfo              *file,
                                                        const gchar                  *text,
-                                                       guint                         index);
+                                                       guint                         idx);
 static void   thunar_sbr_remove_renamer_update        (ThunarSbrRemoveRenamer       *remove_renamer);
 
 
@@ -165,18 +163,18 @@ thunar_sbr_remove_renamer_init (ThunarSbrRemoveRenamer *remove_renamer)
   AtkObject      *object;
   GtkWidget      *combo;
   GtkWidget      *label;
-  GtkWidget      *table;
-  gint            n;
+  GtkWidget      *grid;
+  guint           n;
 
-  table = gtk_table_new (2, 3, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 12);
-  gtk_box_pack_start (GTK_BOX (remove_renamer), table, TRUE, TRUE, 0);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 12);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_box_pack_start (GTK_BOX (remove_renamer), grid, TRUE, TRUE, 0);
+  gtk_widget_show (grid);
 
   label = gtk_label_new_with_mnemonic (_("Remove _From Position:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
   gtk_widget_show (label);
 
   remove_renamer->start_spinner = gtk_spin_button_new_with_range (0u, G_MAXUINT, 1u);
@@ -186,7 +184,7 @@ thunar_sbr_remove_renamer_init (ThunarSbrRemoveRenamer *remove_renamer)
   gtk_spin_button_set_digits (GTK_SPIN_BUTTON (remove_renamer->start_spinner), 0);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (remove_renamer->start_spinner), TRUE);
   gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (remove_renamer->start_spinner), TRUE);
-  gtk_table_attach (GTK_TABLE (table), remove_renamer->start_spinner, 1, 2, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), remove_renamer->start_spinner, 1, 0, 1, 1);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), remove_renamer->start_spinner);
   gtk_widget_show (remove_renamer->start_spinner);
 
@@ -200,18 +198,18 @@ thunar_sbr_remove_renamer_init (ThunarSbrRemoveRenamer *remove_renamer)
   atk_relation_set_add (relations, relation);
   g_object_unref (G_OBJECT (relation));
 
-  combo = gtk_combo_box_new_text ();
+  combo = gtk_combo_box_text_new ();
   klass = g_type_class_ref (THUNAR_SBR_TYPE_OFFSET_MODE);
   for (n = 0; n < klass->n_values; ++n)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(klass->values[n].value_nick));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _(klass->values[n].value_nick));
   exo_mutual_binding_new (G_OBJECT (remove_renamer), "start-offset-mode", G_OBJECT (combo), "active");
-  gtk_table_attach (GTK_TABLE (table), combo, 2, 3, 0, 1, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), combo, 2, 0, 1, 1);
   g_type_class_unref (klass);
   gtk_widget_show (combo);
 
   label = gtk_label_new_with_mnemonic (_("_To Position:"));
-  gtk_misc_set_alignment (GTK_MISC (label), 1.0f, 0.5f);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
+  gtk_label_set_xalign (GTK_LABEL (label), 1.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 1, 1, 1);
   gtk_widget_show (label);
 
   remove_renamer->end_spinner = gtk_spin_button_new_with_range (0u, G_MAXUINT, 1u);
@@ -221,7 +219,7 @@ thunar_sbr_remove_renamer_init (ThunarSbrRemoveRenamer *remove_renamer)
   gtk_spin_button_set_digits (GTK_SPIN_BUTTON (remove_renamer->end_spinner), 0);
   gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (remove_renamer->end_spinner), TRUE);
   gtk_spin_button_set_snap_to_ticks (GTK_SPIN_BUTTON (remove_renamer->end_spinner), TRUE);
-  gtk_table_attach (GTK_TABLE (table), remove_renamer->end_spinner, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), remove_renamer->end_spinner, 1, 1, 1, 1);
   gtk_label_set_mnemonic_widget (GTK_LABEL (label), remove_renamer->end_spinner);
   gtk_widget_show (remove_renamer->end_spinner);
 
@@ -235,12 +233,12 @@ thunar_sbr_remove_renamer_init (ThunarSbrRemoveRenamer *remove_renamer)
   atk_relation_set_add (relations, relation);
   g_object_unref (G_OBJECT (relation));
 
-  combo = gtk_combo_box_new_text ();
+  combo = gtk_combo_box_text_new ();
   klass = g_type_class_ref (THUNAR_SBR_TYPE_OFFSET_MODE);
   for (n = 0; n < klass->n_values; ++n)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (combo), _(klass->values[n].value_nick));
+    gtk_combo_box_text_append_text (GTK_COMBO_BOX_TEXT (combo), _(klass->values[n].value_nick));
   exo_mutual_binding_new (G_OBJECT (remove_renamer), "end-offset-mode", G_OBJECT (combo), "active");
-  gtk_table_attach (GTK_TABLE (table), combo, 2, 3, 1, 2, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), combo, 2, 1, 1, 1);
   g_type_class_unref (klass);
   gtk_widget_show (combo);
 }
@@ -331,7 +329,7 @@ static gchar*
 thunar_sbr_remove_renamer_process (ThunarxRenamer  *renamer,
                                    ThunarxFileInfo *file,
                                    const gchar     *text,
-                                   guint            index)
+                                   guint            idx)
 {
   ThunarSbrRemoveRenamer *remove_renamer = THUNAR_SBR_REMOVE_RENAMER (renamer);
   const gchar            *start_pointer;
@@ -351,7 +349,7 @@ thunar_sbr_remove_renamer_process (ThunarxRenamer  *renamer,
                ? remove_renamer->start_offset : (text_length - remove_renamer->start_offset);
 
   /* check if anything should be removed */
-  if (G_UNLIKELY (start_offset >= end_offset || end_offset >= text_length))
+  if (G_UNLIKELY (start_offset >= end_offset || end_offset > text_length))
     return g_strdup (text);
 
   /* determine start and end pointers */
@@ -376,7 +374,7 @@ thunar_sbr_remove_renamer_update (ThunarSbrRemoveRenamer *remove_renamer)
   guint    end_offset;
 
   /* check if the renamer is realized */
-  if (GTK_WIDGET_REALIZED (remove_renamer))
+  if (gtk_widget_get_realized (GTK_WIDGET (remove_renamer)))
     {
       /* check if start and end offset make sense */
       end_offset = (remove_renamer->end_offset_mode == THUNAR_SBR_OFFSET_MODE_LEFT)
@@ -385,7 +383,7 @@ thunar_sbr_remove_renamer_update (ThunarSbrRemoveRenamer *remove_renamer)
                    ? remove_renamer->start_offset : (G_MAXUINT - remove_renamer->start_offset);
       if (G_UNLIKELY (start_offset >= end_offset))
         {
-          /* if GTK+ wouldn't be that stupid with style properties and 
+          /* if GTK+ wouldn't be that stupid with style properties and
            * type plugins, this would be themable, but unfortunately
            * GTK+ is totally broken, and so it's hardcoded.
            */
@@ -456,7 +454,7 @@ thunar_sbr_remove_renamer_get_end_offset (ThunarSbrRemoveRenamer *remove_renamer
  * Sets the end offset of @remove_renamer to @end_offset.
  **/
 void
-thunar_sbr_remove_renamer_set_end_offset (ThunarSbrRemoveRenamer *remove_renamer,  
+thunar_sbr_remove_renamer_set_end_offset (ThunarSbrRemoveRenamer *remove_renamer,
                                           guint                   end_offset)
 {
   g_return_if_fail (THUNAR_SBR_IS_REMOVE_RENAMER (remove_renamer));
@@ -548,7 +546,7 @@ thunar_sbr_remove_renamer_get_start_offset (ThunarSbrRemoveRenamer *remove_renam
  * Sets the start offset of @remove_renamer to @start_offset.
  **/
 void
-thunar_sbr_remove_renamer_set_start_offset (ThunarSbrRemoveRenamer *remove_renamer,  
+thunar_sbr_remove_renamer_set_start_offset (ThunarSbrRemoveRenamer *remove_renamer,
                                             guint                   start_offset)
 {
   g_return_if_fail (THUNAR_SBR_IS_REMOVE_RENAMER (remove_renamer));
