@@ -35,8 +35,6 @@
 
 #include <thunar-tpa/thunar-tpa-bindings.h>
 
-
-
 typedef struct _ThunarTpaClass ThunarTpaClass;
 typedef struct _ThunarTpa      ThunarTpa;
 
@@ -55,12 +53,8 @@ GType           thunar_tpa_get_type            (void);
 void            thunar_tpa_register_type       (XfcePanelTypeModule *type_module);
 static void     thunar_tpa_finalize            (GObject             *object);
 static void     thunar_tpa_construct           (XfcePanelPlugin     *panel_plugin);
-
-#if defined (LIBXFCE4PANEL_CHECK_VERSION) && LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
 static gboolean thunar_tpa_size_changed        (XfcePanelPlugin     *panel_plugin,
                                                 gint                 size);
-#endif
-
 static void     thunar_tpa_error               (ThunarTpa           *plugin,
                                                 GError              *error);
 static void     thunar_tpa_state               (ThunarTpa           *plugin,
@@ -153,12 +147,7 @@ thunar_tpa_class_init (ThunarTpaClass *klass)
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = thunar_tpa_construct;
-
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
   plugin_class->size_changed = thunar_tpa_size_changed;
-#endif
-#endif
 }
 
 
@@ -181,7 +170,7 @@ thunar_tpa_init (ThunarTpa *plugin)
   gtk_widget_show (plugin->button);
 
   /* setup the image for the trash plugin */
-  plugin->image = xfce_panel_image_new_from_source ("user-trash");
+  plugin->image = gtk_image_new_from_icon_name ("user-trash", GTK_ICON_SIZE_BUTTON);
   gtk_container_add (GTK_CONTAINER (plugin->button), plugin->image);
   gtk_widget_show (plugin->image);
 
@@ -244,12 +233,8 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 {
   ThunarTpa *plugin = THUNAR_TPA (panel_plugin);
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
   /* make the plugin fit a single row */
   xfce_panel_plugin_set_small (panel_plugin, TRUE);
-#endif
-#endif
 
   /* add the "Empty Trash" menu item */
   xfce_panel_plugin_menu_insert_item (panel_plugin, GTK_MENU_ITEM (plugin->mi));
@@ -260,22 +245,26 @@ thunar_tpa_construct (XfcePanelPlugin *panel_plugin)
 
 
 
-#ifdef LIBXFCE4PANEL_CHECK_VERSION
-#if LIBXFCE4PANEL_CHECK_VERSION (4, 9, 0)
 static gboolean
 thunar_tpa_size_changed (XfcePanelPlugin *panel_plugin,
                          gint             size)
 {
-  g_return_val_if_fail (panel_plugin != NULL, FALSE);
+  ThunarTpa *plugin = THUNAR_TPA (panel_plugin);
+  gint       image_size;
 
   /* make the plugin fit a single row */
   size /= xfce_panel_plugin_get_nrows (panel_plugin);
   gtk_widget_set_size_request (GTK_WIDGET (panel_plugin), size, size);
 
+#if LIBXFCE4PANEL_CHECK_VERSION (4,13,0)
+  image_size = xfce_panel_plugin_get_icon_size (panel_plugin);
+#else
+  image_size = size - 2; // fall-back for older panel versions
+#endif
+  gtk_image_set_pixel_size (GTK_IMAGE (plugin->image), image_size);
+
   return TRUE;
 }
-#endif
-#endif
 
 
 
@@ -297,7 +286,7 @@ thunar_tpa_error (ThunarTpa *plugin,
   g_free (tooltip);
 
   /* setup an error plugin */
-  xfce_panel_image_set_from_source (XFCE_PANEL_IMAGE (plugin->image), "stock_dialog-error");
+  gtk_image_set_from_icon_name (GTK_IMAGE (plugin->image), "stock_dialog-error", GTK_ICON_SIZE_BUTTON);
 }
 
 
@@ -310,7 +299,7 @@ thunar_tpa_state (ThunarTpa *plugin,
   gtk_widget_set_tooltip_text (plugin->button, full ? _("Trash contains files") : _("Trash is empty"));
 
   /* setup the appropriate plugin */
-  xfce_panel_image_set_from_source (XFCE_PANEL_IMAGE (plugin->image), full ? "user-trash-full" : "user-trash");
+  gtk_image_set_from_icon_name (GTK_IMAGE (plugin->image), full ? "user-trash-full" : "user-trash", GTK_ICON_SIZE_BUTTON);
 
   /* sensitivity of the menu item */
   gtk_widget_set_sensitive (plugin->mi, full);
@@ -504,8 +493,9 @@ thunar_tpa_trash_changed (DBusGProxy *proxy,
 static void
 thunar_tpa_display_trash (ThunarTpa *plugin)
 {
-  gchar *display_name;
-  gchar *startup_id;
+  gchar     *display_name;
+  gchar     *startup_id;
+  GdkScreen *screen;
 
   g_return_if_fail (THUNAR_IS_TPA (plugin));
 
@@ -517,7 +507,8 @@ thunar_tpa_display_trash (ThunarTpa *plugin)
         dbus_g_proxy_cancel_call (plugin->proxy, plugin->display_trash_call);
 
       /* schedule a new call */
-      display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
+      screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+      display_name = g_strdup (gdk_display_get_name (gdk_screen_get_display (screen)));
       startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
       plugin->display_trash_call = org_xfce_Trash_display_trash_async (plugin->proxy, display_name, startup_id, thunar_tpa_display_trash_reply, plugin);
       g_free (startup_id);
@@ -530,8 +521,9 @@ thunar_tpa_display_trash (ThunarTpa *plugin)
 static void
 thunar_tpa_empty_trash (ThunarTpa *plugin)
 {
-  gchar *display_name;
-  gchar *startup_id;
+  gchar     *display_name;
+  gchar     *startup_id;
+  GdkScreen *screen;
 
   g_return_if_fail (THUNAR_IS_TPA (plugin));
 
@@ -543,7 +535,8 @@ thunar_tpa_empty_trash (ThunarTpa *plugin)
         dbus_g_proxy_cancel_call (plugin->proxy, plugin->empty_trash_call);
 
       /* schedule a new call */
-      display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
+      screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+      display_name = g_strdup (gdk_display_get_name (gdk_screen_get_display (screen)));
       startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
       plugin->empty_trash_call = org_xfce_Trash_empty_trash_async (plugin->proxy, display_name, startup_id, thunar_tpa_empty_trash_reply, plugin);
       g_free (startup_id);
@@ -557,8 +550,9 @@ static gboolean
 thunar_tpa_move_to_trash (ThunarTpa    *plugin,
                           const gchar **uri_list)
 {
-  gchar *display_name;
-  gchar *startup_id;
+  gchar     *display_name;
+  gchar     *startup_id;
+  GdkScreen *screen;
 
   g_return_val_if_fail (THUNAR_IS_TPA (plugin), FALSE);
   g_return_val_if_fail (uri_list != NULL, FALSE);
@@ -572,7 +566,8 @@ thunar_tpa_move_to_trash (ThunarTpa    *plugin,
     dbus_g_proxy_cancel_call (plugin->proxy, plugin->move_to_trash_call);
 
   /* schedule a new call */
-  display_name = gdk_screen_make_display_name (gtk_widget_get_screen (GTK_WIDGET (plugin)));
+  screen = gtk_widget_get_screen (GTK_WIDGET (plugin));
+  display_name = g_strdup (gdk_display_get_name (gdk_screen_get_display (screen)));
   startup_id = g_strdup_printf ("_TIME%d", gtk_get_current_event_time ());
   plugin->move_to_trash_call = org_xfce_Trash_move_to_trash_async (plugin->proxy, uri_list, display_name, startup_id, thunar_tpa_move_to_trash_reply, plugin);
   g_free (startup_id);
