@@ -29,7 +29,9 @@
 #include <X11/Xlib.h>
 #include <X11/Xatom.h>
 
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
+
+#include <xfconf/xfconf.h>
 
 #include "twp-provider.h"
 
@@ -41,10 +43,10 @@
 
 static void   twp_menu_provider_init            (ThunarxMenuProviderIface *iface);
 static void   twp_provider_finalize             (GObject                  *object);
-static GList *twp_provider_get_file_actions     (ThunarxMenuProvider      *menu_provider,
+static GList *twp_provider_get_file_menu_items  (ThunarxMenuProvider      *menu_provider,
                                                  GtkWidget                *window,
                                                  GList                    *files);
-static void   twp_action_set_wallpaper          (GtkAction                *action,
+static void   twp_action_set_wallpaper          (ThunarxMenuItem          *item,
                                                  gpointer                  user_data);
 static gint   twp_get_active_workspace_number   (GdkScreen *screen);
 
@@ -59,7 +61,6 @@ typedef enum
 
 
 static DesktopType desktop_type = DESKTOP_TYPE_NONE;
-static gboolean    _has_xfconf_query = FALSE;
 static gboolean    _has_gconftool = FALSE;
 
 
@@ -86,7 +87,7 @@ THUNARX_DEFINE_TYPE_WITH_CODE (TwpProvider, twp_provider, G_TYPE_OBJECT,
 static void
 twp_menu_provider_init (ThunarxMenuProviderIface *iface)
 {
-  iface->get_file_actions = twp_provider_get_file_actions;
+  iface->get_file_menu_items = twp_provider_get_file_menu_items;
 }
 
 
@@ -105,13 +106,6 @@ twp_provider_init (TwpProvider *twp_provider)
 {
   gchar *program;
 
-  program = g_find_program_in_path ("xfconf-query");
-  if (G_LIKELY (program != NULL))
-    {
-      _has_xfconf_query = TRUE;
-      g_free (program);
-    }
-
   program = g_find_program_in_path ("gconftool-2");
   if (G_LIKELY (program != NULL))
     {
@@ -131,18 +125,19 @@ twp_provider_finalize (GObject *object)
 
 
 static GList*
-twp_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
-                               GtkWidget           *window,
-                               GList               *files)
+twp_provider_get_file_menu_items (ThunarxMenuProvider *menu_provider,
+                                  GtkWidget           *window,
+                                  GList               *files)
 {
-  GtkWidget *action = NULL;
-  GFile     *location;
-  GList     *actions = NULL;
-  gchar      selection_name[100];
-  Atom       xfce_selection_atom;
-  Atom       nautilus_selection_atom;
-  GdkScreen *gdk_screen = gdk_screen_get_default();
-  gint       xscreen = gdk_screen_get_number(gdk_screen);
+  ThunarxMenuItem *item = NULL;
+  GFile           *location;
+  GList           *items = NULL;
+  gchar           *mime_type;
+  gchar            selection_name[100];
+  Atom             xfce_selection_atom;
+  Atom             nautilus_selection_atom;
+  GdkScreen       *gdk_screen = gdk_screen_get_default();
+  gint             xscreen = gdk_screen_get_number(gdk_screen);
 
   desktop_type = DESKTOP_TYPE_NONE;
 
@@ -164,54 +159,49 @@ twp_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
 
       if (!thunarx_file_info_is_directory (files->data))
         {
-          if (thunarx_file_info_has_mime_type (files->data, "image/jpeg")
-              ||thunarx_file_info_has_mime_type (files->data, "image/png")
-              ||thunarx_file_info_has_mime_type (files->data, "image/svg+xml")
-              ||thunarx_file_info_has_mime_type (files->data, "image/svg+xml-compressed"))
-            {
-              action = g_object_new (GTK_TYPE_ACTION,
-                                     "name", "Twp::setwallpaper",
-                                     "icon-name", "background",
-                                     "label", _("Set as wallpaper"),
-                                     NULL);
-              g_signal_connect (action, "activate", G_CALLBACK (twp_action_set_wallpaper), files->data);
+          /* get the mime type of the file */
+          mime_type = thunarx_file_info_get_mime_type (files->data);
 
-              actions = g_list_append (actions, action);
+          if (g_str_has_prefix (mime_type, "image/")
+              && (thunarx_file_info_has_mime_type (files->data, "image/jpeg")
+                  || thunarx_file_info_has_mime_type (files->data, "image/png")
+                  || thunarx_file_info_has_mime_type (files->data, "image/svg+xml")
+                  || thunarx_file_info_has_mime_type (files->data, "image/svg+xml-compressed")))
+            {
+              item = thunarx_menu_item_new ("Twp::setwallpaper", _("Set as wallpaper"), NULL, "preferences-desktop-wallpaper");
+              g_signal_connect (item, "activate", G_CALLBACK (twp_action_set_wallpaper), files->data);
+
+              items = g_list_append (items, item);
             }
+          g_free(mime_type);
         }
     }
 
   g_snprintf(selection_name, 100, XFDESKTOP_SELECTION_FMT, xscreen);
-  xfce_selection_atom = XInternAtom (gdk_display, selection_name, False);
+  xfce_selection_atom = XInternAtom (gdk_x11_get_default_xdisplay(), selection_name, False);
 
-  if ((XGetSelectionOwner(GDK_DISPLAY(), xfce_selection_atom)))
+  if ((XGetSelectionOwner(gdk_x11_get_default_xdisplay(), xfce_selection_atom)))
     {
-      if (_has_xfconf_query)
-          desktop_type = DESKTOP_TYPE_XFCE;
+        desktop_type = DESKTOP_TYPE_XFCE;
     }
   else
     {
       /* FIXME: This is wrong, nautilus WINDOW_ID is not a selection */
       g_snprintf(selection_name, 100, NAUTILUS_SELECTION_FMT);
-      nautilus_selection_atom = XInternAtom (gdk_display, selection_name, False);
-      if((XGetSelectionOwner(GDK_DISPLAY(), nautilus_selection_atom)))
+      nautilus_selection_atom = XInternAtom (gdk_x11_get_default_xdisplay(), selection_name, False);
+      if((XGetSelectionOwner(gdk_x11_get_default_xdisplay(), nautilus_selection_atom)))
       {
           if (_has_gconftool)
               desktop_type = DESKTOP_TYPE_NAUTILUS;
       }
     }
 
-  if ((desktop_type == DESKTOP_TYPE_NONE) && (action != NULL))
-    {
-        /* gtk_widget_set_sensitive (action, FALSE); */
-    }
-
-  return actions;
+  return items;
 }
 
 static void
-twp_action_set_wallpaper (GtkAction *action,
-                          gpointer   user_data)
+twp_action_set_wallpaper (ThunarxMenuItem *item,
+                          gpointer         user_data)
 {
   ThunarxFileInfo *file_info = user_data;
   GdkDisplay      *display = gdk_display_get_default();
@@ -230,6 +220,9 @@ twp_action_set_wallpaper (GtkAction *action,
   gchar           *file_name = NULL;
   gchar           *hostname = NULL;
   gchar           *command;
+  XfconfChannel   *channel;
+  gboolean         is_single_workspace;
+  gint             current_image_style;
 
   if (n_screens > 1)
     screen = gdk_display_get_default_screen (display);
@@ -267,27 +260,41 @@ twp_action_set_wallpaper (GtkAction *action,
     {
       case DESKTOP_TYPE_XFCE:
         g_debug ("set on xfce");
+
+        channel = xfconf_channel_get ("xfce4-desktop");
+
         /* This is the format for xfdesktop before 4.11 */
         image_path_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-path", screen_nr, monitor_nr);
         image_show_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-show", screen_nr, monitor_nr);
         image_style_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/image-style", screen_nr, monitor_nr);
 
-        command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t string -s %s", image_path_prop, escaped_file_name);
-        g_spawn_command_line_async (command, NULL);
-        g_free (command);
+        /* Set the wallpaper and ensure that it's set to show */
+        xfconf_channel_set_string (channel, image_path_prop, file_name);
+        xfconf_channel_set_bool (channel, image_show_prop, TRUE);
 
-        command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t bool -s true", image_show_prop);
-        g_spawn_command_line_async (command, NULL);
-        g_free (command);
-
-        command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t int -s 0", image_style_prop);
-        g_spawn_command_line_async (command, NULL);
-        g_free (command);
+        /* If there isn't a wallpaper style set, then set one */
+        current_image_style = xfconf_channel_get_int (channel, image_style_prop, -1);
+        if (current_image_style == -1)
+          {
+            xfconf_channel_set_int (channel, image_style_prop, 0);
+          }
 
         g_free(image_path_prop);
         g_free(image_show_prop);
         g_free(image_style_prop);
 
+
+        /* Xfdesktop 4.11+ has a concept of a single-workspace-mode where
+         * the same workspace is used for everything but additionally allows
+         * the user to use any current workspace as the single active
+         * workspace, we'll need to check if it is enabled (which by default
+         * it is) and use that. */
+        is_single_workspace = xfconf_channel_get_bool (channel, "/backdrop/single-workspace-mode", TRUE);
+        if (is_single_workspace)
+          {
+            workspace = xfconf_channel_get_int (channel, "/backdrop/single-workspace-number", 0);
+          }
+
         /* This is the format for xfdesktop post 4.11. A workspace number is
          * added and the monitor is referred to name. We set both formats so
          * that it works as the user expects. */
@@ -305,13 +312,14 @@ twp_action_set_wallpaper (GtkAction *action,
             image_style_prop = g_strdup_printf("/backdrop/screen%d/monitor%d/workspace%d/image-style", screen_nr, monitor_nr, workspace);
           }
 
-        command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t string -s %s", image_path_prop, escaped_file_name);
-        g_spawn_command_line_async (command, NULL);
-        g_free (command);
+        xfconf_channel_set_string (channel, image_path_prop, file_name);
 
-        command = g_strdup_printf ("xfconf-query -c xfce4-desktop -p %s --create -t int -s 3", image_style_prop);
-        g_spawn_command_line_async (command, NULL);
-        g_free (command);
+        /* If there isn't a wallpaper style set, then set one */
+        current_image_style = xfconf_channel_get_int (channel, image_style_prop, -1);
+        if (current_image_style == -1)
+          {
+            xfconf_channel_set_int (channel, image_style_prop, 5);
+          }
 
         g_free(image_path_prop);
         g_free(image_style_prop);
