@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -188,12 +188,12 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
   gtk_window_set_title (GTK_WINDOW (dialog), _("Open With"));
 
   /* create the main widget box */
-  vbox = g_object_new (GTK_TYPE_VBOX, "border-width", 6, "spacing", 12, NULL);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), vbox, TRUE, TRUE, 0);
+  vbox = g_object_new (GTK_TYPE_BOX, "orientation", GTK_ORIENTATION_VERTICAL, "border-width", 6, "spacing", 12, NULL);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
   /* create the header box */
-  header = gtk_hbox_new (FALSE, 6);
+  header = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 6);
   gtk_box_pack_start (GTK_BOX (vbox), header, FALSE, FALSE, 0);
   gtk_widget_show (header);
 
@@ -204,14 +204,14 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
 
   /* create the header label */
   dialog->header_label = gtk_label_new ("");
-  gtk_misc_set_alignment (GTK_MISC (dialog->header_label), 0.0f, 0.5f);
+  gtk_label_set_xalign (GTK_LABEL (dialog->header_label), 0.0f);
   gtk_label_set_line_wrap (GTK_LABEL (dialog->header_label), TRUE);
   gtk_widget_set_size_request (dialog->header_label, 350, -1);
   gtk_box_pack_start (GTK_BOX (header), dialog->header_label, FALSE, FALSE, 0);
   gtk_widget_show (dialog->header_label);
 
   /* create the view box */
-  box = gtk_vbox_new (FALSE, 6);
+  box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_box_pack_start (GTK_BOX (vbox), box, TRUE, TRUE, 0);
   gtk_widget_show (box);
 
@@ -242,10 +242,8 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "style", THUNAR_CHOOSER_MODEL_COLUMN_STYLE,
-                                       "style-set", THUNAR_CHOOSER_MODEL_COLUMN_STYLE_SET,
                                        "text", THUNAR_CHOOSER_MODEL_COLUMN_NAME,
                                        "weight", THUNAR_CHOOSER_MODEL_COLUMN_WEIGHT,
-                                       "weight-set", THUNAR_CHOOSER_MODEL_COLUMN_WEIGHT_SET,
                                        NULL);
   gtk_tree_view_append_column (GTK_TREE_VIEW (dialog->tree_view), column);
 
@@ -265,7 +263,7 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
   gtk_widget_show (dialog->custom_expander);
 
   /* create the "Custom command" box */
-  hbox = gtk_hbox_new (FALSE, 2);
+  hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
   gtk_container_add (GTK_CONTAINER (dialog->custom_expander), hbox);
   gtk_widget_show (hbox);
 
@@ -283,16 +281,16 @@ thunar_chooser_dialog_init (ThunarChooserDialog *dialog)
 
   /* create the "Use as default for this kind of file" button */
   dialog->default_button = gtk_check_button_new_with_mnemonic (_("Use as _default for this kind of file"));
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->default_button), TRUE);
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (dialog->default_button), FALSE);
   exo_binding_new (G_OBJECT (dialog), "open", G_OBJECT (dialog->default_button), "visible");
   gtk_box_pack_start (GTK_BOX (box), dialog->default_button, FALSE, FALSE, 0);
   gtk_widget_show (dialog->default_button);
 
   /* add the "Cancel" button */
-  dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+  dialog->cancel_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Cancel"), GTK_RESPONSE_CANCEL);
 
   /* add the "Ok"/"Open" button */
-  dialog->accept_button = gtk_dialog_add_button (GTK_DIALOG (dialog), GTK_STOCK_OK, GTK_RESPONSE_ACCEPT);
+  dialog->accept_button = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_OK"), GTK_RESPONSE_ACCEPT);
   gtk_dialog_set_response_sensitive (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, FALSE);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
@@ -394,13 +392,14 @@ thunar_chooser_dialog_response (GtkDialog *widget,
   GtkTreeIter          iter;
   const gchar         *content_type;
   const gchar         *exec;
-  GAppInfo            *app_info;
+  GAppInfo            *app_info = NULL;
   gboolean             succeed = TRUE;
   GError              *error = NULL;
   gchar               *path;
   gchar               *name;
   gchar               *s;
   GList                list;
+  GdkScreen           *screen;
 
   /* no special processing for non-accept responses */
   if (G_UNLIKELY (response != GTK_RESPONSE_ACCEPT))
@@ -458,12 +457,12 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       /* remember the application as default for these kind of file */
       succeed = g_app_info_set_as_default_for_type (app_info, content_type, &error);
 
-      /* verify that we were successfull */
+      /* verify that we were successful */
       if (G_UNLIKELY (!succeed))
         {
           /* display an error to the user */
-          thunar_dialogs_show_error (GTK_WIDGET (dialog), 
-                                     error, 
+          thunar_dialogs_show_error (GTK_WIDGET (dialog),
+                                     error,
                                      _("Failed to set default application for \"%s\""),
                                      thunar_file_get_display_name (dialog->file));
 
@@ -475,13 +474,23 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       if (G_LIKELY (succeed))
         thunar_file_changed (dialog->file);
     }
+  else
+    {
+      /* simply try to set the app as last used for this type (we do not show any errors here) */
+      if (g_app_info_set_as_last_used_for_type (app_info, content_type, NULL))
+        {
+          /* emit "changed" on the file if we successfully changed the default application */
+          thunar_file_changed (dialog->file);
+        }
+    }
 
   /* check if we should also execute the application */
   if (G_LIKELY (succeed && dialog->open))
     {
       /* create launch context */
-      context = gdk_app_launch_context_new ();
-      gdk_app_launch_context_set_screen (context, gtk_widget_get_screen (GTK_WIDGET (dialog)));
+      screen = gtk_widget_get_screen (GTK_WIDGET (dialog));
+      context = gdk_display_get_app_launch_context (gdk_screen_get_display (screen));
+      gdk_app_launch_context_set_screen (context, screen);
       gdk_app_launch_context_set_timestamp (context, gtk_get_current_event_time ());
 
       /* create fake file list */
@@ -490,9 +499,9 @@ thunar_chooser_dialog_response (GtkDialog *widget,
       if (!g_app_info_launch (app_info, &list, G_APP_LAUNCH_CONTEXT (context), &error))
         {
           /* display an error to the user */
-          thunar_dialogs_show_error (GTK_WIDGET (dialog), 
-                                     error, 
-                                     _("Failed to execute application \"%s\""), 
+          thunar_dialogs_show_error (GTK_WIDGET (dialog),
+                                     error,
+                                     _("Failed to execute application \"%s\""),
                                      g_app_info_get_name (app_info));
 
           /* release the error */
@@ -543,7 +552,6 @@ thunar_chooser_dialog_context_menu (ThunarChooserDialog *dialog,
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GtkTreeIter       iter;
-  GtkWidget        *image;
   GtkWidget        *item;
   GtkWidget        *menu;
   GAppInfo         *app_info;
@@ -564,15 +572,12 @@ thunar_chooser_dialog_context_menu (ThunarChooserDialog *dialog,
   menu = gtk_menu_new ();
 
   /* append the "Remove Launcher" item */
-  item = gtk_image_menu_item_new_with_mnemonic (_("_Remove Launcher"));
+  item = gtk_menu_item_new_with_mnemonic (_("_Remove Launcher"));
   gtk_widget_set_sensitive (item, g_app_info_can_delete (app_info));
   g_signal_connect_swapped (G_OBJECT (item), "activate", G_CALLBACK (thunar_chooser_dialog_action_remove), dialog);
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
   gtk_widget_show (item);
 
-  image = gtk_image_new_from_stock (GTK_STOCK_REMOVE, GTK_ICON_SIZE_MENU);
-  gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
-
   /* run the menu on the dialog's screen (takes over the floating of menu) */
   thunar_gtk_menu_run (GTK_MENU (menu), GTK_WIDGET (dialog), NULL, NULL, button, timestamp);
 
@@ -709,8 +714,8 @@ thunar_chooser_dialog_action_remove (ThunarChooserDialog *dialog)
                                         GTK_BUTTONS_NONE,
                                         _("Are you sure that you want to remove \"%s\"?"), name);
       gtk_dialog_add_buttons (GTK_DIALOG (message),
-                              GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                              GTK_STOCK_REMOVE, GTK_RESPONSE_YES,
+                              _("_Cancel"), GTK_RESPONSE_CANCEL,
+                              _("_Remove"), GTK_RESPONSE_YES,
                               NULL);
       gtk_dialog_set_default_response (GTK_DIALOG (message), GTK_RESPONSE_YES);
       gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message), _("This will remove the application launcher that appears in the file "
@@ -757,8 +762,8 @@ thunar_chooser_dialog_browse_clicked (GtkWidget           *button,
   chooser = gtk_file_chooser_dialog_new (_("Select an Application"),
                                          GTK_WINDOW (dialog),
                                          GTK_FILE_CHOOSER_ACTION_OPEN,
-                                         GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                                         GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+                                         _("_Cancel"), GTK_RESPONSE_CANCEL,
+                                         _("_Open"), GTK_RESPONSE_ACCEPT,
                                          NULL);
   gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
 
@@ -946,7 +951,7 @@ thunar_chooser_dialog_expand (ThunarChooserDialog *dialog)
 
   /* reset the cursor */
   if (G_LIKELY (gtk_widget_get_realized (GTK_WIDGET (dialog))))
-    gdk_window_set_cursor (GTK_WIDGET (dialog)->window, NULL);
+    gdk_window_set_cursor (gtk_widget_get_window (GTK_WIDGET (dialog)), NULL);
 
   /* grab focus to the tree view widget */
   if (G_LIKELY (gtk_widget_get_realized (dialog->tree_view)))
@@ -1159,7 +1164,7 @@ thunar_chooser_dialog_set_open (ThunarChooserDialog *dialog,
   dialog->open = open;
 
   /* change the accept button label text */
-  gtk_button_set_label (GTK_BUTTON (dialog->accept_button), open ? GTK_STOCK_OPEN : GTK_STOCK_OK);
+  gtk_button_set_label (GTK_BUTTON (dialog->accept_button), open ? _("_Open") : _("_OK"));
 
   /* notify listeners */
   g_object_notify (G_OBJECT (dialog), "open");
