@@ -3,18 +3,18 @@
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009-2011 Jannis Pohlmann <jannis@xfce.org>
  *
- * This program is free software; you can redistribute it and/or 
+ * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of 
+ * published by the Free Software Foundation; either version 2 of
  * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
- * You should have received a copy of the GNU General Public 
- * License along with this program; if not, write to the Free 
+ * You should have received a copy of the GNU General Public
+ * License along with this program; if not, write to the Free
  * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
  * Boston, MA 02110-1301, USA.
  */
@@ -52,6 +52,7 @@ enum
 
 enum
 {
+  TARGET_TEXT_URI_LIST,
   TARGET_GNOME_COPIED_FILES,
   TARGET_UTF8_STRING,
 };
@@ -59,6 +60,7 @@ enum
 
 
 static void thunar_clipboard_manager_finalize           (GObject                     *object);
+static void thunar_clipboard_manager_dispose            (GObject                     *object);
 static void thunar_clipboard_manager_get_property       (GObject                     *object,
                                                          guint                        prop_id,
                                                          GValue                      *value,
@@ -117,6 +119,7 @@ typedef struct
 
 static const GtkTargetEntry clipboard_targets[] =
 {
+  { "text/uri-list", 0, TARGET_TEXT_URI_LIST },
   { "x-special/gnome-copied-files", 0, TARGET_GNOME_COPIED_FILES },
   { "UTF8_STRING", 0, TARGET_UTF8_STRING }
 };
@@ -136,6 +139,7 @@ thunar_clipboard_manager_class_init (ThunarClipboardManagerClass *klass)
   GObjectClass *gobject_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
+  gobject_class->dispose = thunar_clipboard_manager_dispose;
   gobject_class->finalize = thunar_clipboard_manager_finalize;
   gobject_class->get_property = thunar_clipboard_manager_get_property;
 
@@ -148,7 +152,7 @@ thunar_clipboard_manager_class_init (ThunarClipboardManagerClass *klass)
    **/
   g_object_class_install_property (gobject_class,
                                    PROP_CAN_PASTE,
-                                   g_param_spec_boolean ("can-paste", "can-pase", "can-paste",
+                                   g_param_spec_boolean ("can-paste", "can-paste", "can-paste",
                                                          FALSE,
                                                          EXO_PARAM_READABLE));
 
@@ -179,6 +183,27 @@ thunar_clipboard_manager_init (ThunarClipboardManager *manager)
 
 
 
+static void
+thunar_clipboard_manager_dispose (GObject *object)
+{
+  ThunarClipboardManager *manager = THUNAR_CLIPBOARD_MANAGER (object);
+
+  /* store the clipboard if we still own it and a clipboard
+   * manager is running (gtk_clipboard_store checks this) */
+  if (gtk_clipboard_get_owner (manager->clipboard) == object
+      && manager->files != NULL)
+    {
+      gtk_clipboard_set_can_store (manager->clipboard, clipboard_targets,
+                                   G_N_ELEMENTS (clipboard_targets));
+
+      gtk_clipboard_store (manager->clipboard);
+    }
+
+  (*G_OBJECT_CLASS (thunar_clipboard_manager_parent_class)->dispose) (object);
+}
+
+
+
 static void
 thunar_clipboard_manager_finalize (GObject *object)
 {
@@ -276,11 +301,11 @@ thunar_clipboard_manager_contents_received (GtkClipboard     *clipboard,
   gchar                       *data;
 
   /* check whether the retrieval worked */
-  if (G_LIKELY (selection_data->length > 0))
+  if (G_LIKELY (gtk_selection_data_get_length (selection_data) > 0))
     {
       /* be sure the selection data is zero-terminated */
-      data = (gchar *) selection_data->data;
-      data[selection_data->length] = '\0';
+      data = (gchar *) gtk_selection_data_get_data (selection_data);
+      data[gtk_selection_data_get_length (selection_data)] = '\0';
 
       /* check whether to copy or move */
       if (g_ascii_strncasecmp (data, "copy\n", 5) == 0)
@@ -316,7 +341,7 @@ thunar_clipboard_manager_contents_received (GtkClipboard     *clipboard,
       if (G_UNLIKELY (!path_copy))
         gtk_clipboard_clear (manager->clipboard);
 
-      /* check the contents of the clipboard again if either the Xserver or 
+      /* check the contents of the clipboard again if either the Xserver or
        * our GTK+ version doesn't support the XFixes extension */
       if (!gdk_display_supports_selection_notification (gtk_clipboard_get_display (manager->clipboard)))
         {
@@ -381,16 +406,53 @@ thunar_clipboard_manager_targets_received (GtkClipboard     *clipboard,
 
 
 
+static gchar *
+thunar_clipboard_manager_g_file_list_to_string (GList       *list,
+                                                const gchar *prefix,
+                                                gboolean     format_for_text,
+                                                gsize       *len)
+{
+  GString *string;
+  gchar   *tmp;
+  GList   *lp;
+
+  /* allocate initial string */
+  string = g_string_new (prefix);
+
+  for (lp = list; lp != NULL; lp = lp->next)
+    {
+      if (format_for_text)
+        tmp = g_file_get_parse_name (G_FILE (lp->data));
+      else
+        tmp = g_file_get_uri (G_FILE (lp->data));
+
+      string = g_string_append (string, tmp);
+      g_free (tmp);
+
+      if (lp->next != NULL)
+        string = g_string_append_c (string, '\n');
+    }
+
+  if (len != NULL)
+    *len = string->len;
+
+  return g_string_free (string, FALSE);
+}
+
+
+
 static void
 thunar_clipboard_manager_get_callback (GtkClipboard     *clipboard,
                                        GtkSelectionData *selection_data,
                                        guint             target_info,
                                        gpointer          user_data)
 {
-  ThunarClipboardManager *manager = THUNAR_CLIPBOARD_MANAGER (user_data);
-  GList                  *file_list = NULL;
-  gchar                  *string_list;
-  gchar                  *data;
+  ThunarClipboardManager  *manager = THUNAR_CLIPBOARD_MANAGER (user_data);
+  GList                   *file_list;
+  gchar                   *str;
+  gchar                  **uris;
+  const gchar             *prefix;
+  gsize                    len;
 
   _thunar_return_if_fail (GTK_IS_CLIPBOARD (clipboard));
   _thunar_return_if_fail (THUNAR_IS_CLIPBOARD_MANAGER (manager));
@@ -399,19 +461,25 @@ thunar_clipboard_manager_get_callback (GtkClipboard     *clipboard,
   /* determine the path list from the file list */
   file_list = thunar_file_list_to_thunar_g_file_list (manager->files);
 
-  /* determine the string representation of the path list */
-  string_list = thunar_g_file_list_to_string (file_list);
-
   switch (target_info)
     {
+    case TARGET_TEXT_URI_LIST:
+      uris = thunar_g_file_list_to_stringv (file_list);
+      gtk_selection_data_set_uris (selection_data, uris);
+      g_strfreev (uris);
+      break;
+
     case TARGET_GNOME_COPIED_FILES:
-      data = g_strconcat (manager->files_cutted ? "cut\n" : "copy\n", string_list, NULL);
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) data, strlen (data));
-      g_free (data);
+      prefix = manager->files_cutted ? "cut\n" : "copy\n";
+      str = thunar_clipboard_manager_g_file_list_to_string (file_list, prefix, FALSE, &len);
+      gtk_selection_data_set (selection_data, gtk_selection_data_get_target (selection_data), 8, (guchar *) str, len);
+      g_free (str);
       break;
 
     case TARGET_UTF8_STRING:
-      gtk_selection_data_set (selection_data, selection_data->target, 8, (guchar *) string_list, strlen (string_list));
+      str = thunar_clipboard_manager_g_file_list_to_string (file_list, NULL, TRUE, &len);
+      gtk_selection_data_set_text (selection_data, str, len);
+      g_free (str);
       break;
 
     default:
@@ -420,7 +488,6 @@ thunar_clipboard_manager_get_callback (GtkClipboard     *clipboard,
 
   /* cleanup */
   thunar_g_file_list_free (file_list);
-  g_free (string_list);
 }
 
 
@@ -533,6 +600,9 @@ thunar_clipboard_manager_get_for_display (GdkDisplay *display)
   g_signal_connect (G_OBJECT (manager->clipboard), "owner-change",
                     G_CALLBACK (thunar_clipboard_manager_owner_changed), manager);
 
+  /* look for usable data on the clipboard */
+  thunar_clipboard_manager_owner_changed (manager->clipboard, NULL, manager);
+
   return manager;
 }
 
