@@ -1,4 +1,4 @@
-/* $Id: thunar-column-editor.c 24510 2007-01-16 19:16:27Z benny $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006-2007 Benedikt Meurer <benny@xfce.org>
  *
@@ -21,6 +21,8 @@
 #include <config.h>
 #endif
 
+#include <libxfce4ui/libxfce4ui.h>
+
 #include <thunar/thunar-abstract-dialog.h>
 #include <thunar/thunar-column-editor.h>
 #include <thunar/thunar-dialogs.h>
@@ -31,8 +33,6 @@
 
 
 
-static void thunar_column_editor_class_init         (ThunarColumnEditorClass  *klass);
-static void thunar_column_editor_init               (ThunarColumnEditor       *column_editor);
 static void thunar_column_editor_finalize           (GObject                  *object);
 static void thunar_column_editor_help_clicked       (GtkWidget                *button,
                                                      ThunarColumnEditor       *column_editor);
@@ -73,36 +73,7 @@ struct _ThunarColumnEditor
 
 
 
-static GObjectClass *thunar_column_editor_parent_class;
-
-
-
-GType
-thunar_column_editor_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarColumnEditorClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunar_column_editor_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarColumnEditor),
-        0,
-        (GInstanceInitFunc) thunar_column_editor_init,
-        NULL,
-      };
-
-      type = g_type_register_static (THUNAR_TYPE_ABSTRACT_DIALOG, I_("ThunarColumnEditor"), &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (ThunarColumnEditor, thunar_column_editor, THUNAR_TYPE_ABSTRACT_DIALOG)
 
 
 
@@ -111,9 +82,6 @@ thunar_column_editor_class_init (ThunarColumnEditorClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunar_column_editor_parent_class = g_type_class_peek_parent (klass);
-
   gobject_class = G_OBJECT_CLASS (klass);
   gobject_class->finalize = thunar_column_editor_finalize;
 }
@@ -132,7 +100,7 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   GtkWidget         *frame;
   GtkWidget         *image;
   GtkWidget         *label;
-  GtkWidget         *table;
+  GtkWidget         *grid;
   GtkWidget         *vbox;
   GtkWidget         *swin;
 
@@ -147,22 +115,21 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
                          column_editor, NULL, G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 
   /* setup the dialog */
-  gtk_dialog_add_button (GTK_DIALOG (column_editor), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
+  gtk_dialog_add_button (GTK_DIALOG (column_editor), _("_Close"), GTK_RESPONSE_CLOSE);
   gtk_dialog_set_default_response (GTK_DIALOG (column_editor), GTK_RESPONSE_CLOSE);
-  gtk_dialog_set_has_separator (GTK_DIALOG (column_editor), FALSE);
   gtk_window_set_resizable (GTK_WINDOW (column_editor), FALSE);
   gtk_window_set_title (GTK_WINDOW (column_editor), _("Configure Columns in the Detailed List View"));
 
   /* add the "Help" button */
-  button = gtk_button_new_from_stock (GTK_STOCK_HELP);
+  button = gtk_button_new_with_mnemonic (_("_Help"));
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_help_clicked), column_editor);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (column_editor)->action_area), button, FALSE, FALSE, 0);
-  gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (GTK_DIALOG (column_editor)->action_area), button, TRUE);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG (column_editor))), button, FALSE, FALSE, 0);
+  gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (gtk_dialog_get_action_area (GTK_DIALOG (column_editor))), button, TRUE);
   gtk_widget_show (button);
 
-  vbox = gtk_vbox_new (FALSE, 6);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 6);
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (column_editor)->vbox), vbox, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (column_editor))), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
   frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
@@ -174,24 +141,27 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   gtk_frame_set_label_widget (GTK_FRAME (frame), label);
   gtk_widget_show (label);
 
-  table = gtk_table_new (8, 2, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 12);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   /* create the top label for the column editor dialog */
   label = gtk_label_new (_("Choose the order of information to appear in the\ndetailed list view."));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1, GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_widget_set_hexpand (label, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 2, 1);
   gtk_widget_show (label);
 
   /* create the scrolled window for the tree view */
   swin = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swin), GTK_SHADOW_IN);
-  gtk_table_attach (GTK_TABLE (table), swin, 0, 1, 1, 7, GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
+  gtk_widget_set_hexpand (swin, TRUE);
+  gtk_widget_set_vexpand (swin, TRUE);
+  gtk_grid_attach (GTK_GRID (grid), swin, 0, 1, 1, 6);
   gtk_widget_show (swin);
 
   /* create the tree view */
@@ -225,44 +195,46 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   /* create the "Move Up" button */
   column_editor->up_button = gtk_button_new_with_mnemonic (_("Move _Up"));
   g_signal_connect (G_OBJECT (column_editor->up_button), "clicked", G_CALLBACK (thunar_column_editor_move_up), column_editor);
-  gtk_table_attach (GTK_TABLE (table), column_editor->up_button, 1, 2, 1, 2, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), column_editor->up_button, 1, 1, 1, 1);
   gtk_widget_show (column_editor->up_button);
 
-  image = gtk_image_new_from_stock (GTK_STOCK_GO_UP, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name ("go-up-symbolic", GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_always_show_image (GTK_BUTTON (column_editor->up_button), TRUE);
   gtk_button_set_image (GTK_BUTTON (column_editor->up_button), image);
   gtk_widget_show (image);
 
   /* create the "Move Down" button */
   column_editor->down_button = gtk_button_new_with_mnemonic (_("Move Dow_n"));
   g_signal_connect (G_OBJECT (column_editor->down_button), "clicked", G_CALLBACK (thunar_column_editor_move_down), column_editor);
-  gtk_table_attach (GTK_TABLE (table), column_editor->down_button, 1, 2, 2, 3, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), column_editor->down_button, 1, 2, 1, 1);
   gtk_widget_show (column_editor->down_button);
 
-  image = gtk_image_new_from_stock (GTK_STOCK_GO_DOWN, GTK_ICON_SIZE_BUTTON);
+  image = gtk_image_new_from_icon_name ("go-down-symbolic", GTK_ICON_SIZE_BUTTON);
+  gtk_button_set_always_show_image (GTK_BUTTON (column_editor->down_button), TRUE);
   gtk_button_set_image (GTK_BUTTON (column_editor->down_button), image);
   gtk_widget_show (image);
 
   /* create the "Show" button */
   column_editor->show_button = gtk_button_new_with_mnemonic (_("_Show"));
   g_signal_connect (G_OBJECT (column_editor->show_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
-  gtk_table_attach (GTK_TABLE (table), column_editor->show_button, 1, 2, 3, 4, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), column_editor->show_button, 1, 3, 1, 1);
   gtk_widget_show (column_editor->show_button);
 
   /* create the "Hide" button */
   column_editor->hide_button = gtk_button_new_with_mnemonic (_("Hi_de"));
   g_signal_connect (G_OBJECT (column_editor->hide_button), "clicked", G_CALLBACK (thunar_column_editor_toggle_visibility), column_editor);
-  gtk_table_attach (GTK_TABLE (table), column_editor->hide_button, 1, 2, 4, 5, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), column_editor->hide_button, 1, 4, 1, 1);
   gtk_widget_show (column_editor->hide_button);
 
   /* create the horiz separator */
-  separator = gtk_hseparator_new ();
-  gtk_table_attach (GTK_TABLE (table), separator, 1, 2, 5, 6, GTK_FILL, 0, 0, 0);
+  separator = gtk_separator_new (GTK_ORIENTATION_HORIZONTAL);
+  gtk_grid_attach (GTK_GRID (grid), separator, 1, 5, 1, 1);
   gtk_widget_show (separator);
 
   /* create the "Use Default" button */
   button = gtk_button_new_with_mnemonic (_("Use De_fault"));
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (thunar_column_editor_use_defaults), column_editor);
-  gtk_table_attach (GTK_TABLE (table), button, 1, 2, 6, 7, GTK_FILL, 0, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), button, 1, 6, 1, 1);
   gtk_widget_show (button);
 
   frame = g_object_new (GTK_TYPE_FRAME, "border-width", 0, "shadow-type", GTK_SHADOW_NONE, NULL);
@@ -274,26 +246,26 @@ thunar_column_editor_init (ThunarColumnEditor *column_editor)
   gtk_frame_set_label_widget (GTK_FRAME (frame), label);
   gtk_widget_show (label);
 
-  table = gtk_table_new (2, 1, FALSE);
-  gtk_table_set_row_spacings (GTK_TABLE (table), 6);
-  gtk_table_set_col_spacings (GTK_TABLE (table), 6);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 12);
-  gtk_container_add (GTK_CONTAINER (frame), table);
-  gtk_widget_show (table);
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+  gtk_grid_set_row_spacing (GTK_GRID (grid), 6);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 12);
+  gtk_container_add (GTK_CONTAINER (frame), grid);
+  gtk_widget_show (grid);
 
   /* create the label that explains the column sizing option */
   label = gtk_label_new (_("By default columns will be automatically expanded if\n"
                            "needed to ensure the text is fully visible. If you dis-\n"
                            "able this behavior below the file manager will always\n"
                            "use the user defined column widths."));
-  gtk_misc_set_alignment (GTK_MISC (label), 0.0f, 0.5f);
-  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_label_set_xalign (GTK_LABEL (label), 0.0f);
+  gtk_grid_attach (GTK_GRID (grid), label, 0, 0, 1, 1);
   gtk_widget_show (label);
 
   /* create the "Automatically expand columns as needed" button */
   button = gtk_check_button_new_with_mnemonic (_("Automatically _expand columns as needed"));
   exo_mutual_binding_new_with_negation (G_OBJECT (column_editor->preferences), "last-details-view-fixed-columns", G_OBJECT (button), "active");
-  gtk_table_attach (GTK_TABLE (table), button, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
+  gtk_grid_attach (GTK_GRID (grid), button, 0, 1, 1, 1);
   thunar_gtk_label_set_a11y_relation (GTK_LABEL (label), button);
   gtk_widget_show (button);
 
@@ -336,7 +308,10 @@ thunar_column_editor_help_clicked (GtkWidget          *button,
   _thunar_return_if_fail (GTK_IS_BUTTON (button));
 
   /* open the user manual */
-  thunar_dialogs_show_help (column_editor, "the-file-manager-window", "visible-columns-in-the-detailed-list-view");
+  xfce_dialog_show_help (GTK_WINDOW (gtk_widget_get_toplevel (button)),
+                         "thunar",
+                         "the-file-manager-window",
+                         "customizing_the_appearance");
 }
 
 
@@ -474,7 +449,7 @@ thunar_column_editor_update_buttons (ThunarColumnEditor *column_editor)
   GtkTreeIter       iter;
   gboolean          mutable;
   gboolean          visible;
-  gint              index;
+  gint              idx;
 
   /* determine the selected row */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (column_editor->tree_view));
@@ -486,9 +461,9 @@ thunar_column_editor_update_buttons (ThunarColumnEditor *column_editor)
         return;
 
       /* update the "Move Up"/"Move Down" buttons */
-      index = gtk_tree_path_get_indices (path)[0];
-      gtk_widget_set_sensitive (column_editor->up_button, (index > 0));
-      gtk_widget_set_sensitive (column_editor->down_button, (index + 1 < gtk_tree_model_iter_n_children (model, NULL)));
+      idx = gtk_tree_path_get_indices (path)[0];
+      gtk_widget_set_sensitive (column_editor->up_button, (idx > 0));
+      gtk_widget_set_sensitive (column_editor->down_button, (idx + 1 < gtk_tree_model_iter_n_children (model, NULL)));
 
       /* update the "Show"/"Hide" buttons */
       gtk_tree_model_get (model, &iter, THUNAR_COLUMN_MODEL_COLUMN_MUTABLE, &mutable, THUNAR_COLUMN_MODEL_COLUMN_VISIBLE, &visible, -1);
@@ -523,7 +498,7 @@ thunar_column_editor_use_defaults (GtkWidget          *button,
   GtkTreeSelection *selection;
   GParamSpec       *pspec;
   GValue            value = { 0, };
-  gint              n;
+  guint             n;
 
   /* reset the given properties to its default values */
   for (n = 0; n < G_N_ELEMENTS (PROPERTY_NAMES); ++n)
@@ -542,21 +517,6 @@ thunar_column_editor_use_defaults (GtkWidget          *button,
 
 
 
-/**
- * thunar_column_editor_new:
- *
- * Allocates a new #ThunarColumnEditor instance.
- *
- * Return value: the newly allocated #ThunarColumnEditor.
- **/
-GtkWidget*
-thunar_column_editor_new (void)
-{
-  return g_object_new (THUNAR_TYPE_COLUMN_EDITOR, NULL);
-}
-
-
-
 /**
  * thunar_show_column_editor:
  * @parent : the #GtkWidget the #GdkScreen on which to open the
@@ -575,7 +535,7 @@ thunar_show_column_editor (gpointer parent)
 {
   GtkWidget *window = NULL;
   GtkWidget *dialog;
-  GdkScreen *screen;
+  GdkScreen *screen = NULL;
 
   _thunar_return_if_fail (parent == NULL || GDK_IS_SCREEN (parent) || GTK_IS_WIDGET (parent));
 
@@ -598,10 +558,10 @@ thunar_show_column_editor (gpointer parent)
     }
 
   /* display the column editor */
-  dialog = thunar_column_editor_new ();
+  dialog = g_object_new (THUNAR_TYPE_COLUMN_EDITOR, NULL);
 
   /* check if we have a toplevel window */
-  if (G_LIKELY (window != NULL && GTK_WIDGET_TOPLEVEL (window)))
+  if (G_LIKELY (window != NULL && gtk_widget_get_toplevel (window)))
     {
       /* dialog is transient for toplevel window and modal */
       gtk_window_set_destroy_with_parent (GTK_WINDOW (dialog), TRUE);
@@ -609,6 +569,10 @@ thunar_show_column_editor (gpointer parent)
       gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
     }
 
+  /* set the screen for the window */
+  if (screen != NULL && GDK_IS_SCREEN (screen))
+    gtk_window_set_screen (GTK_WINDOW (dialog), screen);
+
   /* run the dialog */
   gtk_dialog_run (GTK_DIALOG (dialog));
 
