@@ -28,7 +28,6 @@
 #include <thunar/thunar-details-view-ui.h>
 #include <thunar/thunar-gtk-extensions.h>
 #include <thunar/thunar-private.h>
-#include <thunar/thunar-text-renderer.h>
 #include <thunar/thunar-preferences.h>
 
 
@@ -237,11 +236,11 @@ thunar_details_view_init (ThunarDetailsView *details_view)
                           G_CALLBACK (thunar_details_view_row_changed), details_view);
 
   /* allocate the shared right-aligned text renderer */
-  right_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 1.0f, NULL);
+  right_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 1.0f, NULL);
   g_object_ref_sink (G_OBJECT (right_aligned_renderer));
 
   /* allocate the shared left-aligned text renderer */
-  left_aligned_renderer = g_object_new (THUNAR_TYPE_TEXT_RENDERER, "xalign", 0.0f, NULL);
+  left_aligned_renderer = g_object_new (GTK_TYPE_CELL_RENDERER_TEXT, "xalign", 0.0f, NULL);
   g_object_ref_sink (G_OBJECT (left_aligned_renderer));
 
   /* allocate the tree view columns */
@@ -276,6 +275,7 @@ thunar_details_view_init (ThunarDetailsView *details_view)
 
           /* add some spacing between the icon and the name */
           gtk_tree_view_column_set_spacing (details_view->columns[column], 2);
+          gtk_tree_view_column_set_expand (details_view->columns[column], TRUE);
         }
       else
         {
@@ -439,7 +439,7 @@ thunar_details_view_get_selected_items (ThunarStandardView *standard_view)
 
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), NULL);
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   return gtk_tree_selection_get_selected_rows (selection, NULL);
 }
 
@@ -452,7 +452,7 @@ thunar_details_view_select_all (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_select_all (selection);
 }
 
@@ -465,7 +465,7 @@ thunar_details_view_unselect_all (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_unselect_all (selection);
 }
 
@@ -494,7 +494,7 @@ thunar_details_view_selection_invert (ThunarStandardView *standard_view)
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
 
   /* block updates */
   g_signal_handlers_block_by_func (selection, thunar_standard_view_selection_changed, standard_view);
@@ -529,7 +529,7 @@ thunar_details_view_select_path (ThunarStandardView *standard_view,
 
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (GTK_BIN (standard_view)->child));
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))));
   gtk_tree_selection_select_path (selection, path);
 }
 
@@ -546,17 +546,17 @@ thunar_details_view_set_cursor (ThunarStandardView *standard_view,
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
   /* make sure the name renderer is editable */
-  mode = standard_view->name_renderer->mode;
-  standard_view->name_renderer->mode = GTK_CELL_RENDERER_MODE_EDITABLE;
+  g_object_get ( G_OBJECT (standard_view->name_renderer), "mode", &mode, NULL);
+  g_object_set ( G_OBJECT (standard_view->name_renderer), "mode", GTK_CELL_RENDERER_MODE_EDITABLE, NULL);
 
   /* tell the tree view to start editing the given row */
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), 0);
-  gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (GTK_BIN (standard_view)->child),
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), 0);
+  gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))),
                                     path, column, standard_view->name_renderer,
                                     start_editing);
 
   /* reset the name renderer mode */
-  standard_view->name_renderer->mode = mode;
+  g_object_set (G_OBJECT (standard_view->name_renderer), "mode", mode, NULL);
 }
 
 
@@ -573,8 +573,8 @@ thunar_details_view_scroll_to_path (ThunarStandardView *standard_view,
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
 
   /* tell the tree view to scroll to the given row */
-  column = gtk_tree_view_get_column (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), 0);
-  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), path, column, use_align, row_align, col_align);
+  column = gtk_tree_view_get_column (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), 0);
+  gtk_tree_view_scroll_to_cell (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), path, column, use_align, row_align, col_align);
 }
 
 
@@ -588,7 +588,7 @@ thunar_details_view_get_path_at_pos (ThunarStandardView *standard_view,
 
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), NULL);
 
-  if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), x, y, &path, NULL))
+  if (gtk_tree_view_get_dest_row_at_pos (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), x, y, &path, NULL))
     return path;
 
   return NULL;
@@ -603,7 +603,7 @@ thunar_details_view_get_visible_range (ThunarStandardView *standard_view,
 {
   _thunar_return_val_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view), FALSE);
 
-  return gtk_tree_view_get_visible_range (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), start_path, end_path);
+  return gtk_tree_view_get_visible_range (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), start_path, end_path);
 }
 
 
@@ -613,7 +613,7 @@ thunar_details_view_highlight_path (ThunarStandardView *standard_view,
                                     GtkTreePath        *path)
 {
   _thunar_return_if_fail (THUNAR_IS_DETAILS_VIEW (standard_view));
-  gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (GTK_BIN (standard_view)->child), path, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
+  gtk_tree_view_set_drag_dest_row (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (standard_view))), path, GTK_TREE_VIEW_DROP_INTO_OR_AFTER);
 }
 
 
@@ -662,7 +662,7 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
   GtkTreePath       *path = NULL;
   GtkTreeIter        iter;
   GtkTreeViewColumn *column;
-  GtkTreeViewColumn *first_column;
+  GtkTreeViewColumn *name_column;
   ThunarFile        *file;
   GtkAction         *action;
   ThunarPreferences *preferences;
@@ -676,8 +676,8 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
   /* get the current selection */
   selection = gtk_tree_view_get_selection (tree_view);
 
-  /* get the first column of the tree view */
-  first_column = gtk_tree_view_get_column (tree_view, 0);
+  /* get the column showing the filenames */
+  name_column = details_view->columns[THUNAR_COLUMN_NAME];
 
   /* unselect all selected items if the user clicks on an empty area
    * of the treeview and no modifier key is active */
@@ -698,12 +698,12 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
         {
           gtk_tree_path_free (cursor_path);
 
-          if (column != first_column)
+          if (column != name_column)
             gtk_tree_selection_unselect_all (selection);
 
-          /* do not start rubber banding from the first column */
+          /* do not start rubber banding from the name column */
           if (!gtk_tree_selection_path_is_selected (selection, path)
-              && column == first_column)
+              && column == name_column)
             {
               /* the clicked row does not have the focus and is not
                * selected, so make it the new selection (start) */
@@ -729,7 +729,7 @@ thunar_details_view_button_press_event (GtkTreeView       *tree_view,
         }
       else
         {
-          if (column != first_column)
+          if (column != name_column)
             {
               /* if the clicked path is not selected, unselect all other paths */
               if (!gtk_tree_selection_path_is_selected (selection, path))
@@ -808,7 +808,7 @@ thunar_details_view_key_press_event (GtkTreeView       *tree_view,
                                      ThunarDetailsView *details_view)
 {
   /* popup context menu if "Menu" or "<Shift>F10" is pressed */
-  if (event->keyval == GDK_Menu || ((event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_F10))
+  if (event->keyval == GDK_KEY_Menu || ((event->state & GDK_SHIFT_MASK) != 0 && event->keyval == GDK_KEY_F10))
     {
       thunar_standard_view_context_menu (THUNAR_STANDARD_VIEW (details_view), 0, event->time);
       return TRUE;
@@ -879,7 +879,7 @@ thunar_details_view_columns_changed (ThunarColumnModel *column_model,
       /* change the order of the column relative to its predecessor */
       if (G_LIKELY (column > 0))
         {
-          gtk_tree_view_move_column_after (GTK_TREE_VIEW (GTK_BIN (details_view)->child),
+          gtk_tree_view_move_column_after (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (details_view))),
                                            details_view->columns[column_order[column]],
                                            details_view->columns[column_order[column - 1]]);
         }
@@ -985,7 +985,7 @@ thunar_details_view_set_fixed_columns (ThunarDetailsView *details_view,
       /* for fixed columns mode, we can enable the fixed height
        * mode to improve the performance of the GtkTreeVeiw.
        */
-      gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (GTK_BIN (details_view)->child), fixed_columns);
+      gtk_tree_view_set_fixed_height_mode (GTK_TREE_VIEW (gtk_bin_get_child (GTK_BIN (details_view))), fixed_columns);
 
       /* notify listeners */
       g_object_notify (G_OBJECT (details_view), "fixed-columns");
