@@ -237,6 +237,11 @@ thunar_g_file_get_display_name (GFile *file)
           display_name = g_strdup (_("File System"));
           g_free (base_name);
         }
+      else if (strcmp (base_name, "Trash") == 0)
+        {
+          display_name = g_strdup (_("Trash"));
+          g_free (base_name);
+        }
       else if (g_utf8_validate (base_name, -1, NULL))
        {
          display_name = base_name;
@@ -512,7 +517,19 @@ thunar_g_file_list_to_stringv (GList *list)
   uris = g_new0 (gchar *, g_list_length (list) + 1);
 
   for (lp = list, n = 0; lp != NULL; lp = lp->next)
-    uris[n++] = g_file_get_uri (G_FILE (lp->data));
+    {
+      /* Prefer native paths for interoperability. */
+      gchar *path = g_file_get_path (G_FILE (lp->data));
+      if (path == NULL)
+        {
+          uris[n++] = g_file_get_uri (G_FILE (lp->data));
+        }
+      else
+        {
+          uris[n++] = g_filename_to_uri (path, NULL, NULL);
+          g_free(path);
+        }
+    }
 
   return uris;
 }
@@ -526,9 +543,12 @@ thunar_g_app_info_launch (GAppInfo          *info,
                           GAppLaunchContext *context,
                           GError           **error)
 {
-  gboolean result = FALSE;
-  gchar   *new_path = NULL;
-  gchar   *old_path = NULL;
+  ThunarFile   *file;
+  GList        *lp;
+  const gchar  *content_type;
+  gboolean      result = FALSE;
+  gchar        *new_path = NULL;
+  gchar        *old_path = NULL;
 
   _thunar_return_val_if_fail (G_IS_APP_INFO (info), FALSE);
   _thunar_return_val_if_fail (working_directory == NULL || G_IS_FILE (working_directory), FALSE);
@@ -554,6 +574,25 @@ thunar_g_app_info_launch (GAppInfo          *info,
   /* launch the paths with the specified app info */
   result = g_app_info_launch (info, path_list, context, error);
 
+  /* if successful, remember the application as last used for the file types */
+  if (result == TRUE)
+    {
+      for (lp = path_list; lp != NULL; lp = lp->next)
+        {
+          file = thunar_file_get (lp->data, NULL);
+          if (file != NULL)
+            {
+              content_type = thunar_file_get_content_type (file);
+
+              /* emit "changed" on the file if we successfully changed the last used application */
+              if (g_app_info_set_as_last_used_for_type (info, content_type, NULL))
+                thunar_file_changed (file);
+
+              g_object_unref (file);
+            }
+        }
+    }
+
   /* check if we need to reset the working directory to the one Thunar was
    * opened from */
   if (old_path != NULL)
