@@ -185,9 +185,7 @@ static void               thunar_list_model_set_date_style        (ThunarListMod
 static gint               thunar_list_model_get_num_files         (ThunarListModel        *store);
 static gboolean           thunar_list_model_get_folders_first     (ThunarListModel        *store);
 
-static gboolean           thunar_list_model_get_file_size_binary  (ThunarListModel        *store);
-static void               thunar_list_model_set_file_size_binary  (ThunarListModel        *store,
-                                                                   gboolean                file_size_binary);
+
 
 struct _ThunarListModelClass
 {
@@ -1093,7 +1091,7 @@ thunar_list_model_sort (ThunarListModel *store)
     new_order[g_sequence_iter_get_position (old_order[n])] = n;
 
   /* tell the view about the new item order */
-  path = gtk_tree_path_new_root ();
+  path = gtk_tree_path_new_first ();
   gtk_tree_model_rows_reordered (GTK_TREE_MODEL (store), path, NULL, new_order);
   gtk_tree_path_free (path);
 
@@ -1170,7 +1168,7 @@ thunar_list_model_file_changed (ThunarFileMonitor *file_monitor,
                 }
 
               /* tell the view about the new item order */
-              path = gtk_tree_path_new_root ();
+              path = gtk_tree_path_new_first ();
               gtk_tree_model_rows_reordered (GTK_TREE_MODEL (store), path, NULL, new_order);
               gtk_tree_path_free (path);
 
@@ -1679,6 +1677,12 @@ thunar_list_model_set_case_sensitive (ThunarListModel *store,
 
       /* notify listeners */
       g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_CASE_SENSITIVE]);
+
+      /* emit a "changed" signal for each row, so the display is
+         reloaded with the new case-sensitive setting */
+      gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                              (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                              NULL);
     }
 }
 
@@ -1883,6 +1887,12 @@ thunar_list_model_set_folders_first (ThunarListModel *store,
   store->sort_folders_first = folders_first;
   g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FOLDERS_FIRST]);
   thunar_list_model_sort (store);
+
+  /* emit a "changed" signal for each row, so the display is
+     reloaded with the new folders first setting */
+  gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                          (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                          NULL);
 }
 
 
@@ -1999,7 +2009,7 @@ thunar_list_model_set_show_hidden (ThunarListModel *store,
  *
  * Return value: %TRUE if file size format is binary.
  **/
-static gboolean
+gboolean
 thunar_list_model_get_file_size_binary (ThunarListModel *store)
 {
   _thunar_return_val_if_fail (THUNAR_IS_LIST_MODEL (store), FALSE);
@@ -2016,7 +2026,7 @@ thunar_list_model_get_file_size_binary (ThunarListModel *store)
  * If @file_size_binary is %TRUE the file size should be
  * formatted as binary.
  **/
-static void
+void
 thunar_list_model_set_file_size_binary (ThunarListModel *store,
                                         gboolean         file_size_binary)
 {
@@ -2036,6 +2046,12 @@ thunar_list_model_set_file_size_binary (ThunarListModel *store,
 
       /* notify listeners */
       g_object_notify_by_pspec (G_OBJECT (store), list_model_props[PROP_FILE_SIZE_BINARY]);
+
+      /* emit a "changed" signal for each row, so the display is
+         reloaded with the new binary file size setting */
+      gtk_tree_model_foreach (GTK_TREE_MODEL (store),
+                              (GtkTreeModelForeachFunc) gtk_tree_model_row_changed,
+                              NULL);
     }
 }
 
