@@ -27,11 +27,11 @@
 
 #include <glib-object.h>
 
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
 #define THUNARX_MAJOR_VERSION 1
-#define THUNARX_MINOR_VERSION 6
-#define THUNARX_MICRO_VERSION 18
+#define THUNARX_MINOR_VERSION 7
+#define THUNARX_MICRO_VERSION 0
 
 #define THUNARX_CHECK_VERSION(major,minor,micro) \
   (THUNARX_MAJOR_VERSION > (major) \
@@ -58,6 +58,6 @@ const gchar *thunarx_check_version (guint required_major,
 #endif /* __GNUC__ */
 #endif /* !defined(G_GNUC_WARN_UNUSED_RESULT) */
 
-G_END_DECLS;
+G_END_DECLS
 
 #endif /* !__THUNARX_CONFIG_H__ */
