@@ -1,4 +1,4 @@
-/* $Id$ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2009 Jannis Pohlmann <jannis@xfce.org>
@@ -40,6 +40,15 @@ enum
 
 static guint file_info_signals[LAST_SIGNAL];
 
+/**
+ * SECTION: thunarx-file-info
+ * @short_description: Abstraction of a file handled within the file manager
+ * @title: ThunarxFileInfo
+ * @include: thunarx/thunarx.h
+ *
+ * The <interface>ThunarxFileInfo</interface> interface provides extensions with
+ * a way to access information about a file handled within the file manager.
+ */
 
 
 GType
@@ -59,7 +68,7 @@ thunarx_file_info_get_type (void)
                                             0);
 
       g_type_interface_add_prerequisite (type, G_TYPE_OBJECT);
-      
+
       /**
        * ThunarxFileInfo::changed:
        * @file_info : a #ThunarxFileInfo.
@@ -290,8 +299,8 @@ thunarx_file_info_is_directory (ThunarxFileInfo *file_info)
  *
  * Returns the #GFileInfo associated with @file_info,
  * which includes additional information about the @file_info
- * as queried from GIO earlier. The caller is responsible to free the 
- * returned #GFileInfo object using g_object_unref() when 
+ * as queried from GIO earlier. The caller is responsible to free the
+ * returned #GFileInfo object using g_object_unref() when
  * no longer needed.
  *
  * Return value: the #GFileInfo object associated with @file_info,
@@ -311,12 +320,12 @@ thunarx_file_info_get_file_info (ThunarxFileInfo *file_info)
  * @file_info : a #ThunarxFileInfo.
  *
  * Returns the #GFileInfo which includes additional information about
- * the filesystem @file_info resides on. The caller is responsible to 
- * free the returned #GFileInfo object using g_object_unref() when 
+ * the filesystem @file_info resides on. The caller is responsible to
+ * free the returned #GFileInfo object using g_object_unref() when
  * no longer needed.
  *
  * Return value: the #GFileInfo containing information about the
- *               filesystem of @file_info or %NULL if no filesystem 
+ *               filesystem of @file_info or %NULL if no filesystem
  *               information is available. It MUST be released using
  *               g_object_unref().
  **/
@@ -418,16 +427,16 @@ thunarx_file_info_list_get_type (void)
 GList*
 thunarx_file_info_list_copy (GList *file_infos)
 {
-  GList *list = NULL;
-  GList *lp;
+#if GLIB_CHECK_VERSION (2, 34, 0)
+  return g_list_copy_deep (file_infos, (GCopyFunc) g_object_ref, NULL);
+#else
+  GList *copy;
 
-  if (file_infos != NULL)
-    {
-      for (lp = g_list_last (file_infos); lp != NULL; lp = lp->prev)
-        list = g_list_prepend (list, g_object_ref (G_OBJECT (lp->data)));
-    }
+  copy = g_list_copy (file_infos);
+  g_list_foreach (copy, (GFunc) g_object_ref, NULL);
 
-  return list;
+  return copy;
+#endif
 }
 
 
