@@ -1,4 +1,4 @@
-/* $Id: thunarx-private.h 26421 2007-12-02 13:46:28Z benny $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -27,19 +27,17 @@
 
 #include <glib-object.h>
 
-G_BEGIN_DECLS;
+G_BEGIN_DECLS
 
-#if GLIB_CHECK_VERSION(2,9,0)
 #define I_(string) (g_intern_static_string ((string)))
-#else
-#define I_(string) (g_quark_to_string (g_quark_from_static_string ((string))))
-#endif
 
+G_GNUC_INTERNAL
 void   thunarx_object_list_take_reference (GList      *object_list,
-                                           gpointer    target) G_GNUC_INTERNAL;
+                                           gpointer    target);
 
-gchar *thunarx_param_spec_get_option_name (GParamSpec *pspec) G_GNUC_INTERNAL G_GNUC_MALLOC;
+G_GNUC_INTERNAL
+gchar *thunarx_param_spec_get_option_name (GParamSpec *pspec) G_GNUC_MALLOC;
 
-G_END_DECLS;
+G_END_DECLS
 
 #endif /* !__THUNARX_PRIVATE_H__ */
