@@ -1,4 +1,4 @@
-/* $Id: thunarx-renamer.c 26421 2007-12-02 13:46:28Z benny $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2005-2006 Benedikt Meurer <benny@xfce.org>
  *
@@ -26,7 +26,6 @@
 
 #include <thunarx/thunarx-renamer.h>
 #include <thunarx/thunarx-private.h>
-#include <thunarx/thunarx-alias.h>
 
 
 
@@ -50,32 +49,56 @@ enum
 };
 
 
+/**
+ * SECTION: thunarx-renamer
+ * @short_description: The abstract base class for bulk renamers
+ * @title: ThunarxRenamer
+ * @include: thunarx/thunarx.h
+ * @image: bulk-rename.png
+ *
+ * The abstract base class <type>ThunarxRenamer</type> is implemented by
+ * extensions which provide additional bulk renamers that should be used in
+ * the bulk rename dialog.
+ *
+ * Derived classes must override the thunarx_renamer_process() method, which
+ * is called by the bulk rename dialog for every file to generate a new name. For
+ * example, the <type>ThunarSbrReplaceRenamer</type> class included in the
+ * <literal>thunar-sbr</literal> plugin (which is part of the Thunar distribution)
+ * provides a bulk renamer, named <emphasis>Search &amp; Replace</emphasis>,
+ * which allows the user to rename multiple files by searching for a pattern in
+ * each file name and, if the pattern is found, replacing it with the specified
+ * replacement text.
+ *
+ * The active <type>ThunarxRenamer</type>s user interface is displayed in a frame
+ * below the file list, as shown in the screenshot above. Derived classes should try
+ * to limit the number of widgets displayed in the main user interface. For example,
+ * if you have more than six settings, you should consider adding an <guibutton>Advanced</guibutton>,
+ * button which opens a dialog with the additional settings.
+ */
 
-static void     thunarx_renamer_class_init        (ThunarxRenamerClass    *klass);
-static void     thunarx_renamer_init              (ThunarxRenamer         *renamer);
-static void     thunarx_renamer_finalize          (GObject                *object);
-static GObject *thunarx_renamer_constructor       (GType                   type,
-                                                   guint                   n_construct_properties,
-                                                   GObjectConstructParam  *construct_properties);
-static void     thunarx_renamer_get_property      (GObject                *object,
-                                                   guint                   prop_id,
-                                                   GValue                 *value,
-                                                   GParamSpec             *pspec);
-static void     thunarx_renamer_set_property      (GObject                *object,
-                                                   guint                   prop_id,
-                                                   const GValue           *value,
-                                                   GParamSpec             *pspec);
-static gchar   *thunarx_renamer_real_process      (ThunarxRenamer         *renamer,
-                                                   ThunarxFileInfo        *file,
-                                                   const gchar            *text,
-                                                   guint                   num);
-static void     thunarx_renamer_real_load         (ThunarxRenamer         *renamer,
-                                                   GHashTable             *settings);
-static void     thunarx_renamer_real_save         (ThunarxRenamer         *renamer,
-                                                   GHashTable             *settings);
-static GList   *thunarx_renamer_real_get_actions  (ThunarxRenamer         *renamer,
-                                                   GtkWindow              *window,
-                                                   GList                  *files);
+static void     thunarx_renamer_finalize             (GObject                *object);
+static GObject *thunarx_renamer_constructor          (GType                   type,
+                                                      guint                   n_construct_properties,
+                                                      GObjectConstructParam  *construct_properties);
+static void     thunarx_renamer_get_property         (GObject                *object,
+                                                      guint                   prop_id,
+                                                      GValue                 *value,
+                                                      GParamSpec             *pspec);
+static void     thunarx_renamer_set_property         (GObject                *object,
+                                                      guint                   prop_id,
+                                                      const GValue           *value,
+                                                      GParamSpec             *pspec);
+static gchar   *thunarx_renamer_real_process         (ThunarxRenamer         *renamer,
+                                                      ThunarxFileInfo        *file,
+                                                      const gchar            *text,
+                                                      guint                   num);
+static void     thunarx_renamer_real_load            (ThunarxRenamer         *renamer,
+                                                      GHashTable             *settings);
+static void     thunarx_renamer_real_save            (ThunarxRenamer         *renamer,
+                                                      GHashTable             *settings);
+static GList   *thunarx_renamer_real_get_menu_items  (ThunarxRenamer         *renamer,
+                                                      GtkWindow              *window,
+                                                      GList                  *files);
 
 
 
@@ -87,37 +110,11 @@ struct _ThunarxRenamerPrivate
 
 
 
-static GObjectClass *thunarx_renamer_parent_class;
-static guint         renamer_signals[LAST_SIGNAL];
+static guint renamer_signals[LAST_SIGNAL];
 
 
 
-GType
-thunarx_renamer_get_type (void)
-{
-  static GType type = G_TYPE_INVALID;
-
-  if (G_UNLIKELY (type == G_TYPE_INVALID))
-    {
-      static const GTypeInfo info =
-      {
-        sizeof (ThunarxRenamerClass),
-        NULL,
-        NULL,
-        (GClassInitFunc) thunarx_renamer_class_init,
-        NULL,
-        NULL,
-        sizeof (ThunarxRenamer),
-        0,
-        (GInstanceInitFunc) thunarx_renamer_init,
-        NULL,
-      };
-
-      type = g_type_register_static (GTK_TYPE_VBOX, I_("ThunarxRenamer"), &info, G_TYPE_FLAG_ABSTRACT);
-    }
-
-  return type;
-}
+G_DEFINE_ABSTRACT_TYPE (ThunarxRenamer, thunarx_renamer, GTK_TYPE_BOX)
 
 
 
@@ -126,9 +123,6 @@ thunarx_renamer_class_init (ThunarxRenamerClass *klass)
 {
   GObjectClass *gobject_class;
 
-  /* determine the parent type class */
-  thunarx_renamer_parent_class = g_type_class_peek_parent (klass);
-
   /* add private data */
   g_type_class_add_private (klass, sizeof (ThunarxRenamerPrivate));
 
@@ -141,7 +135,7 @@ thunarx_renamer_class_init (ThunarxRenamerClass *klass)
   klass->process = thunarx_renamer_real_process;
   klass->load = thunarx_renamer_real_load;
   klass->save = thunarx_renamer_real_save;
-  klass->get_actions = thunarx_renamer_real_get_actions;
+  klass->get_menu_items = thunarx_renamer_real_get_menu_items;
 
   /**
    * ThunarxRenamer:help-url:
@@ -208,7 +202,8 @@ thunarx_renamer_init (ThunarxRenamer *renamer)
   /* grab a pointer on the private data */
   renamer->priv = THUNARX_RENAMER_GET_PRIVATE (renamer);
 
-  /* initialize the GtkVBox to sane defaults */
+  /* initialize the GtkBox to sane defaults */
+  gtk_orientable_set_orientation (GTK_ORIENTABLE (renamer), GTK_ORIENTATION_VERTICAL);
   gtk_container_set_border_width (GTK_CONTAINER (renamer), 12);
   gtk_box_set_homogeneous (GTK_BOX (renamer), FALSE);
   gtk_box_set_spacing (GTK_BOX (renamer), 6);
@@ -339,7 +334,7 @@ thunarx_renamer_real_load (ThunarxRenamer *renamer,
 
       /* determine the key for the parameter */
       key = thunarx_param_spec_get_option_name (specs[n]);
-      
+
       /* check if we have a value for that key */
       setting = g_hash_table_lookup (settings, key);
       if (G_LIKELY (setting != NULL))
@@ -411,9 +406,9 @@ thunarx_renamer_real_save (ThunarxRenamer *renamer,
 
 
 static GList*
-thunarx_renamer_real_get_actions (ThunarxRenamer *renamer,
-                                  GtkWindow      *window,
-                                  GList          *files)
+thunarx_renamer_real_get_menu_items (ThunarxRenamer *renamer,
+                                     GtkWindow      *window,
+                                     GList          *files)
 {
   /* return an empty list, derived classes may override this method */
   return NULL;
@@ -527,7 +522,7 @@ thunarx_renamer_set_name (ThunarxRenamer *renamer,
  *            determined.
  * @text    : the part of the filename to which the
  *            @renamer should be applied.
- * @index   : the index of the file in the list, used
+ * @idx     : the index of the file in the list, used
  *            for renamers that work on numbering.
  *
  * Determines the replacement for @text (which is the relevant
@@ -543,12 +538,12 @@ gchar*
 thunarx_renamer_process (ThunarxRenamer  *renamer,
                          ThunarxFileInfo *file,
                          const gchar     *text,
-                         guint            index)
+                         guint            idx)
 {
   g_return_val_if_fail (THUNARX_IS_FILE_INFO (file), NULL);
   g_return_val_if_fail (THUNARX_IS_RENAMER (renamer), NULL);
   g_return_val_if_fail (g_utf8_validate (text, -1, NULL), NULL);
-  return (*THUNARX_RENAMER_GET_CLASS (renamer)->process) (renamer, file, text, index);
+  return (*THUNARX_RENAMER_GET_CLASS (renamer)->process) (renamer, file, text, idx);
 }
 
 
@@ -627,20 +622,20 @@ thunarx_renamer_save (ThunarxRenamer *renamer,
 
 
 /**
- * thunarx_renamer_get_actions:
+ * thunarx_renamer_get_menu_items:
  * @renamer : a #ThunarxRenamer.
  * @window  : a #GtkWindow or %NULL.
  * @files   : a #GList of #ThunarxFileInfo<!---->s.
  *
- * Returns the list of #GtkAction<!---->s provided by @renamer for
+ * Returns the list of #ThunarxMenuItem<!---->s provided by @renamer for
  * the given list of @files. By default, this method returns %NULL
  * (the empty list), but derived classes may override this method
- * to provide additional actions for files in the bulk renamer
+ * to provide additional items for files in the bulk renamer
  * dialog list.
- * 
- * The returned #GtkAction<!---->s will be displayed in the file's
+ *
+ * The returned #ThunarxMenuItem<!---->s will be displayed in the file's
  * context menu of the bulk renamer dialog, when this @renamer is
- * active. For example, an ID3-Tag based renamer may add an action
+ * active. For example, an ID3-Tag based renamer may add an menu item
  * "Edit Tags" to the context menus of supported media files and,
  * when activated, display a dialog (which should be transient and
  * modal for @window, if not %NULL), which allows the users to edit
@@ -648,44 +643,43 @@ thunarx_renamer_save (ThunarxRenamer *renamer,
  *
  * Derived classes that override this method should always check
  * first if all the #ThunarxFileInfo<!---->s in the list of @files
- * are supported, and only return actions that can be performed on
+ * are supported, and only return menu items that can be performed on
  * this specific list of @files. For example, the ID3-Tag renamer
  * mentioned above, should first check whether all items in @files
  * are actually audio files. The thunarx_file_info_has_mime_type()
  * of the #ThunarxFileInfo interface can be used to easily test
  * whether a file in the @files list is of a certain MIME type.
  *
- * Some actions may only work properly if only a single file ist
+ * Some menu items may only work properly if only a single file is
  * selected (for example, the ID3-Tag renamer will probably only
  * supporting editing one file at a time). In this case you have
  * basicly two options: Either you can return %NULL here if @files
- * does not contain exactly one item, or you can return the actions
+ * does not contain exactly one item, or you can return the menu items
  * as usual, but make them insensitive, using:
  * <informalexample><programlisting>
- * gtk_action_set_sensitive (action, FALSE);
+ * thunarx_menu_item_set_sensitive (item, FALSE);
  * </programlisting></informalexample>
  * The latter has the advantage that the user will still notice the
- * existance of the action and probably realize that it can only be
+ * existance of the menu item and probably realize that it can only be
  * applied to a single item at once.
  *
  * The caller is responsible to free the returned list using something
  * like the following:
  * <informalexample><programlisting>
- * g_list_foreach (list, (GFunc) g_object_unref, NULL);
- * g_list_free (list);
+ * g_list_free_full (list, g_object_unref);
  * </programlisting></informalexample>
  *
  * As a special note, this method automatically takes a reference on the
- * @renamer for every #GtkAction object returned from the real implementation
+ * @renamer for every #ThunarxMenuItem object returned from the real implementation
  * of this method in @renamer. This is to make sure that the extension stays
- * in memory for atleast the time that the actions are used.
+ * in memory for at least the time that the menu items are used.
  *
- * The #GtkAction<!---->s returned from this method must be namespaced with
- * the module to avoid collision with internal file manager actions and
- * actions provided by other extensions. For example, the menu action
+ * The name of #ThunarxMenuItem<!---->s returned from this method must be namespaced with
+ * the module to avoid collision with internal file manager menu items and
+ * menu items provided by other extensions. For example, the menu item
  * provided by the ID3-Tag renamer mentioned above, should be named
  * <literal>TagRenamer::edit-tags</literal> (if <literal>TagRenamer</literal>
- * is the class name). For additional information about the way #GtkAction<!---->s
+ * is the class name). For additional information about the way #ThunarxMenuItem<!---->s
  * should be returned from extensions and the way they are used, read the
  * description of the #ThunarxMenuProvider interface or read the introduction
  * provided with this reference manual.
@@ -696,28 +690,28 @@ thunarx_renamer_save (ThunarxRenamer *renamer,
  * Instead, if @window is not %NULL, add a weak reference using the
  * g_object_weak_ref() or g_object_add_weak_pointer() method. But don't
  * forget to release the weak reference if @window survived the lifetime
- * of your action (which is likely to be the case in most situations).
+ * of your menu item (which is likely to be the case in most situations).
  *
- * Return value: the list of #GtkAction<!---->s provided by @renamer
+ * Return value: the list of #ThunarxMenuItem<!---->s provided by @renamer
  *               for the given list of @files.
  **/
 GList*
-thunarx_renamer_get_actions (ThunarxRenamer *renamer,
-                             GtkWindow      *window,
-                             GList          *files)
+thunarx_renamer_get_menu_items (ThunarxRenamer *renamer,
+                                GtkWindow      *window,
+                                GList          *files)
 {
-  GList *actions;
+  GList *items;
 
   g_return_val_if_fail (THUNARX_IS_RENAMER (renamer), NULL);
   g_return_val_if_fail (window == NULL || GTK_IS_WINDOW (window), NULL);
 
-  /* query the actions from the implementation */
-  actions = (*THUNARX_RENAMER_GET_CLASS (renamer)->get_actions) (renamer, window, files);
+  /* query the menu items from the implementation */
+  items = (*THUNARX_RENAMER_GET_CLASS (renamer)->get_menu_items) (renamer, window, files);
 
-  /* take a reference on the renamer for each action */
-  thunarx_object_list_take_reference (actions, renamer);
+  /* take a reference on the renamer for each menu item */
+  thunarx_object_list_take_reference (items, renamer);
 
-  return actions;
+  return items;
 }
 
 
@@ -737,8 +731,3 @@ thunarx_renamer_changed (ThunarxRenamer *renamer)
   g_return_if_fail (THUNARX_IS_RENAMER (renamer));
   g_signal_emit (G_OBJECT (renamer), renamer_signals[CHANGED], 0);
 }
-
-
-
-#define __THUNARX_RENAMER_C__
-#include <thunarx/thunarx-aliasdef.c>
